/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

struct allinone {
  uma n;
  void *prev,*first;
} A;

void new_allinone_rec(const char *dir) {
  DIR *dd;
  struct dirent *entry;
  struct stat buf;
  void *p;
  uma base_len;

  dd = opendir(dir);
  if ( !dd ) return;
  base_len = strlen(dir);
  chdir(dir);
  for( ; (entry = readdir(dd)) ; ) {
    stat(entry->d_name,&buf);
    if ( !hiddenfile(entry->d_name) ) {
      if ( S_ISDIR(buf.st_mode) ) {
        if ( entry->d_name[0] != '.' || entry->d_name[1] != '.' ||
             entry->d_name[2] != 0 ) {
          char *s = malloc2(base_len + strlen(entry->d_name) + 2);
          sprintf(s,"%s/%s",dir,entry->d_name);
          new_allinone_rec(s);
          free(s);
        }
      }
      else {
        uma len;
        A.n++;
        len = base_len + strlen(entry->d_name) + 2;
        p = malloc2(sizeof(void*) + len);
        *(void**)p = 0;
        if ( A.prev ) *(void**)A.prev = p; else A.first = p;
        A.prev = p;
        p = (void**)p + 1;
        sprintf((char*)p,"%s/%s",dir,entry->d_name);
      }
    }
  }
  closedir(dd);
  return;
}

void allinone_output(list *l) {
  uma i;
  int y,m;
  m = G.height - 1;
  for( y = 0,i = l->first ; y < m && i <= l->last ; y++,i++ ) {
    G.term.attr = 0;
    if ( i == l->bold ) G.term.attr |= CCT_BOLD;
    if ( i == l->sel ) G.term.attr |= CCT_REVERSE;
    cct_hline(&G.term,0,y,' ',G.width);
    print_path_name(0,y,l->item[i],G.width);
  }
}

void allinone_play(list *l,uma i) {
  play(l->item[i]);
  l->bold = i;
  G.curr = l;
}

void allinone_keypress(list *l,int c) {
  switch(c) {
    case '\n':
      if ( G.shuffle ) random_init();
      allinone_play(l,l->sel);
      return;
    default:
      list_keypress(l,c);
      return;
  }
}

void allinone_kill(list *l) {
  free(l->path);
  killdir(l->item,l->nf);
}

list *new_allinone(const char *dir) {
  void *q;
  char **i,**p;
  list *n;

  A.first = 0;
  A.prev = 0;
  A.n = 0;
  new_allinone_rec(dir);
  chdir(G.base_dir);
  if ( !A.first ) {
    set_error(ERR_OPEN_DIR);
    return 0;
  }
  i = malloc2(A.n * sizeof(char*));
  p = i;
  for( q = A.first ; q ; q = *(void**)q,i++ ) {
    *i = (char*)((void**)q + 1);
  }
  qsort(p,A.n,sizeof(char*),alphasort2);
  n = malloc2(sizeof(list));
  n->path = strdup2(dir);
  n->item = p;
  n->first = 0;
  n->last = A.n - 1;
  n->sel = 0;
  n->bold = -1;
  n->nf = A.n;
  n->nd = 0;
  n->create = new_allinone;
  n->kill = allinone_kill;
  n->keypress = allinone_keypress;
  n->output = allinone_output;
  n->play = allinone_play;
  return n;
}

