/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void browser_kill(list *l) {
  free(l->path);
  killdir(l->item,l->last + 1);
}

void browser_play(list *l,uma i) {
  char *s;
  if ( i < 0 || i >= l->nf ) return;
  s = malloc2(strlen(l->path) + strlen(l->item[i]) + 2);
  sprintf(s,"%s/%s",l->path,l->item[i] + 1);
  play(s);
  l->bold = i;
  G.curr = l;
  free(s);
}

void browser_keypress(list *l,int c) {
  switch(c) {
    case '\n':
    case 'f': {
      char *s,*f;
      f = l->item[l->sel];
      if ( f[0] == ' ' ) {
        if ( G.shuffle ) random_init();
        browser_play(l,l->sel);
        return;
      }
      if ( !strcmp(l->path,"/") ) {
        s = strdup2(f);
      }
      else {
        s = malloc2(strlen(l->path) + strlen(f) + 1);
        sprintf(s,"%s%s",l->path,f);
      }
      switch(c) {
        list *n;
        case '\n':
          n = new_browser(s);
          if ( IS_VALID_PTR(n) ) list_new(n);
          break;
        case 'f':
          n = new_allinone(s);
          if ( IS_VALID_PTR(n) ) list_new(n);
          break;
      }
      free(s);
      return;
    }
    case 'F': {
      list *n;
      char *s;
      s = strdup2(l->path);
      n = new_allinone(s);
      if ( IS_VALID_PTR(n) ) list_new(n);
      return;
    }
    case 'u': {
      uma i;
      char *s;
      if ( !strcmp(l->path,"/") ) return;
      s = strdup2(l->path);
      for( i = strlen(s) - 1 ; s[i] != '/' ; i-- );
      if ( i == 0 ) s[i + 1] = 0; else s[i] = 0;
      list_new(new_browser(s));
      free(s);
      return;
    }
    default:
      list_keypress(l,c);
      return;
  }
}

void browser_output(list *l) {
  uma i;
  int y,m;
  m = G.height - 1;
  for( y = 0,i = l->first ; y < m && i <= l->last ; y++,i++ ) {
    G.term.attr = 0;
    if ( i == l->bold ) G.term.attr |= CCT_BOLD;
    if ( i == l->sel ) G.term.attr |= CCT_REVERSE;
    cct_hline(&G.term,0,y,' ',G.width);
    cct_nputs(&G.term,0,y,l->item[i],G.width);
  }
}

uma browser_scandir(char ***entries,uma *nf,uma *nd,const char *dir) {
  DIR *dd = opendir(dir);
  struct dirent *entry;
  struct stat buf;
  void *p,*previous,*q = 0; /* for gcc */
  char **i;
  uma n,nf2,nd2;
  
  n = 0,nf2 = 0,nd2 = 0;
  if ( !dd ) return -1;
  chdir(dir);
  for( previous = 0 ; (entry = readdir(dd)) ; ) {
    stat(entry->d_name,&buf);
    if ( !hiddenfile(entry->d_name) ) {
      n++;
      p = malloc2(sizeof(void*) + strlen(entry->d_name) + 2);
      *(void**)p = 0;
      if ( previous ) *(void**)previous = p; else q = p;
      previous = p;
      p = (void**)p + 1;
      if ( S_ISDIR(buf.st_mode) ) {
        *(char*)p = '/';
        nd2++;
      }
      else {
        *(char*)p = ' ';
        nf2++;
      }
      p = (char*)p + 1;
      strcpy((char*)p,entry->d_name);
    }
  }
  chdir(G.base_dir);
  closedir(dd);
  if ( n == 0 ) return -2;
  i = malloc2(n * sizeof(char*));
  p = i;
  for( ; q ; q = *(void**)q,i++ ) *i = (char*)((void**)q + 1);
  qsort(p,n,sizeof(char*),alphasort2);
  *entries = (char**)p;
  *nf = nf2;
  *nd = nd2;
  return 0;
}

list *new_browser(const char *path) {
  char **entries;
  uma nf,nd,ret;
  list *l;
  ret = browser_scandir(&entries,&nf,&nd,path);
  if ( ret < 0 ) {
    if ( ret == -1 ) {
      set_error(ERR_OPEN_DIR);
      return NULL;
    }
    if ( ret == -2 ) {
      set_error(ERR_DIR_EMPTY);
      return (void*)-1;
    }
  }
  l = malloc2(sizeof(list));
  l->item = entries;
  l->first = 0;
  l->last = nf + nd - 1;
  l->sel = 0;
  l->bold = -1;
  l->nf = nf;
  l->nd = nd;
  l->path = strdup2(path);
  l->create = new_browser;
  l->kill = browser_kill;
  l->keypress = browser_keypress;
  l->output = browser_output;
  l->play = browser_play;
  return l;
}

