/*
 * Copyright (C) 2010 Guillaume Quintin.
 * This file is part of cct.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

static int _flush(buffer *b) {
  int r,w;
  if ( b->bp <= 0 ) {
    b->bp = 0;
    return 0;
  }
  for( r = b->bp ; r > 0 ; ) {
    w = write(b->fd,&(b->buffer[b->bp - r]),r);
    if ( w == -1 && errno != EINTR ) return -1;
    r -= w;
  }
  b->bp = 0;
  return 0;
}

static int _puts(buffer *b,const char *s) {
  for( ; s[0] ; s++ ) {
    if ( b->bp >= MAX ) {
      if ( _flush(b) == -1 ) return -1;
    }
    b->buffer[b->bp] = s[0];
    b->bp++;
  }
  return 0;
}

static int _putc(buffer *b,char c) {
  if ( b->bp >= MAX ) {
    if ( _flush(b) == -1 ) return -1;
  }
  b->buffer[b->bp] = c;
  b->bp++;
  return 0;
}

static int _put_rune(buffer *b,char c,short attr,short oattr) {
  char a,s[] = "\033[0;.;.;.;.;3.;4.m";
                /* 0123456789 */
  int i;

  if ( oattr != attr ) {
    s[3] = 'm';
    s[4] = 0;
    i = 3;
    if ( attr & CCT_BOLD ) {
      s[i] = ';'; i++;
      s[i] = '1'; i++;
      s[i] = 'm';
      s[i + 1] = 0;
    }
    if ( attr & CCT_UNDERLINE ) {
      s[i] = ';'; i++;
      s[i] = '4'; i++;
      s[i] = 'm';
      s[i + 1] = 0;
    }
    if ( attr & CCT_BLINK ) {
      s[i] = ';'; i++;
      s[i] = '5'; i++;
      s[i] = 'm';
      s[i + 1] = 0;
    }
    if ( attr & CCT_REVERSE ) {
      s[i] = ';'; i++;
      s[i] = '7'; i++;
      s[i] = 'm';
      s[i + 1] = 0;
    }
    a = (char)((attr & 0x00F0) >> 4);
    if ( a ) {
      s[i] = ';'; i++;
      s[i] = '3'; i++;
      s[i] = a + '0' - 1; i++;
      s[i] = 'm';
      s[i + 1] = 0;
    }
    a = (char)((attr & 0x0F00) >> 8);
    if ( a ) {
      s[i] = ';'; i++;
      s[i] = '4'; i++;
      s[i] = a + '0' - 1; i++;
      s[i] = 'm';
      s[i + 1] = 0;
    }
    if ( _puts(b,s) == -1 ) return -1;
  }
  if ( _putc(b,c) == -1 ) return -1;
  return 0;
}

