/*
 * Copyright (C) 2010 Guillaume Quintin.
 * This file is part of cct.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef _CCT_H
#define _CCT_H

#include <termios.h>

/* rune attribute */
#define CCT_NONE            0
#define CCT_BOLD            1
#define CCT_UNDERLINE       2
#define CCT_BLINK           4
#define CCT_REVERSE         8
/* foreground colors */
#define CCT_FORE_BLACK      (1 << 4)
#define CCT_FORE_RED        (2 << 4)
#define CCT_FORE_GREEN      (3 << 4)
#define CCT_FORE_YELLOW     (4 << 4)
#define CCT_FORE_BLUE       (5 << 4)
#define CCT_FORE_MAGENTA    (6 << 4)
#define CCT_FORE_CYAN       (7 << 4)
#define CCT_FORE_WHITE      (8 << 4)
/* background colors */
#define CCT_BACK_BLACK      (1 << 8)
#define CCT_BACK_RED        (2 << 8)
#define CCT_BACK_GREEN      (3 << 8)
#define CCT_BACK_YELLOW     (4 << 8)
#define CCT_BACK_BLUE       (5 << 8)
#define CCT_BACK_MAGENTA    (6 << 8)
#define CCT_BACK_CYAN       (7 << 8)
#define CCT_BACK_WHITE      (8 << 8)
/* */
#define CCT_KEY_UP          0x100
#define CCT_KEY_DOWN        0x101
#define CCT_KEY_RIGHT       0x102
#define CCT_KEY_LEFT        0x103

typedef struct terminal terminal;
struct terminal {
  int fd,w,h,max;
  void *ptr;
  char *C;
  short *A;
  short attr;
  char unknown;
  struct termios old_termios;
};

typedef struct cct_textbox cct_textbox;
struct cct_textbox {
  terminal *t;
  int x,y,w,maxlen;
  char *text;
  int first,last,sel,max;
};

/* term.c */
extern int cct_init_term(terminal*,int);
extern int cct_end_term(terminal*);
extern int cct_resize(terminal*);

/* drawing.c */
extern void cct_putc(terminal*,int,int,char);
extern void cct_puts(terminal*,int,int,const char*);
extern void cct_nputs(terminal*,int,int,const char*,int);
extern void cct_vline(terminal*,int,int,char,int);
extern void cct_hline(terminal*,int,int,char,int);
extern void cct_fill(terminal*,char);
extern void cct_clear(terminal*);

/* input.c */
extern char *cct_input(int);
extern char *cct_helper_print_input(const char*);
extern int cct_get_key(const char*);

/* output.c */
extern int cct_refresh(terminal*);
extern int cct_write(terminal*,int);
extern void cct_col_left_puts(terminal*,int,int,const char*,int);
extern void cct_col_right_puts(terminal*,int,int,const char*,int);

/* utils.c */
extern int cct_attr_on(terminal*,int);
extern int cct_icanon_on(terminal*);
extern int cct_echo_on(terminal*);
extern int cct_attr_off(terminal*,int);
extern int cct_icanon_off(terminal*);
extern int cct_echo_off(terminal*);

/* textbox.c */
extern int cct_textbox_init(cct_textbox*,terminal*,int,int,int,int);
extern void cct_textbox_kill(cct_textbox*);
extern void cct_textbox_resize(cct_textbox*,int,int,int);
extern void cct_textbox_clear(cct_textbox*);
extern void cct_textbox_home(cct_textbox*);
extern void cct_textbox_end(cct_textbox*);
extern void cct_textbox_next(cct_textbox*);
extern void cct_textbox_prev(cct_textbox*);
extern void cct_textbox_del(cct_textbox*);
extern void cct_textbox_bs(cct_textbox*);
extern int cct_textbox_insert(cct_textbox*,const char*);
extern void cct_textbox_draw(cct_textbox*);
extern int cct_textbox_keypress(cct_textbox*,int);

#endif

