/*
 * Copyright (C) 2010 Guillaume Quintin.
 * This file is part of cct.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

static void _draw(terminal *t,int i,char c) {
  t->C[i] = c;
  t->A[i] = t->attr;
}

void cct_putc(terminal *t,int x,int y,char c) {
  if ( x < 0 || x >= t->w || y < 0 || y >= t->h ) return;
  if ( c < 0 ) c = t->unknown;
  _draw(t,y * t->w + x,c);
}

void cct_puts(terminal *t,int x,int y,const char *s) {
  int i;
  if ( x < 0 || x >= t->w || y < 0 || y >= t->h ) return;
  for( i = y * t->w + x ; s[0] && i < t->max ; i++,s++ ) {
    _draw(t,i,(s[0] < 0 ? t->unknown : s[0]));
  }
}

void cct_nputs(terminal *t,int x,int y,const char *s,int n) {
  int i;
  if ( x < 0 || x >= t->w || y < 0 || y >= t->h ) return;
  i = y * t->w + x;
  for( ; s[0] && i < t->max && n > 0 ; i++,n--,s++ ) {
    _draw(t,i,(s[0] < 0 ? t->unknown : s[0]));
  }
}

void cct_vline(terminal *t,int x,int y,char c,int n) {
  int i;
  if ( x < 0 || x >= t->w || y < 0 || y >= t->h ) return;
  if ( c < 0 ) c = t->unknown;
  i = y * t->w + x;
  for( ; n > 0 && i < t->max ; i += t->w,n-- ) {
    _draw(t,i,c);
  }
}

void cct_hline(terminal *t,int x,int y,char c,int n) {
  int i;
  if ( x < 0 || x >= t->w || y < 0 || y >= t->h ) return;
  if ( c < 0 ) c = t->unknown;
  for( i = y * t->w + x ; n > 0 && i < t->max ; i++,n-- ) {
    _draw(t,i,c);
  }
}

void cct_fill(terminal *t,char c) {
  int i;
  if ( c < 0 ) c = t->unknown;
  for( i = 0 ; i < t->max ; i++ ) {
    _draw(t,i,c);
  }
}

void cct_clear(terminal *t) {
  short oattr;
  oattr = t->attr;
  t->attr = 0;
  cct_fill(t,' ');
  t->attr = oattr;
}

void cct_col_left_puts(terminal *t,int x,int y,const char *s,int n) {
  int i;
  if ( x < 0 || x >= t->w || y < 0 || y >= t->h ) return;
  i = y * t->w + x;
  for( ; s[0] && n > 0 && i < t->max ; i++,n--,s++ ) {
    _draw(t,i,s[0]);
  }
  for( ; n > 0 && i < t->max ; i++,n-- ) {
    _draw(t,i,' ');
  }
}

void cct_col_right_puts(terminal *t,int x,int y,const char *s,int n) {
  int i,j;
  if ( x < 0 || x + n - 1 >= t->w || y < 0 || y >= t->h ) return;
  i = y * t->w + x + n - 1;
  j = strlen(s) - 1;
  for( ; j >= 0 && n > 0 && i > 0 ; i--,n--,j-- ) {
    _draw(t,i,s[j]);
  }
  for( ; n > 0 && i > 0 ; i--,n-- ) {
    _draw(t,i,' ');
  }
}

