/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <sys/wait.h>
#include <sys/select.h>
#include "cct/cct.h"

#define VERSION        "cocoju version 2.3"
#define ESC             27
#ifndef WAIT_ANY
#define WAIT_ANY       -1
#endif
#ifndef SIGWINCH
#define SIGWINCH        28
#endif
#define DEV_NULL       "/dev/null"
#define LAST_RAND_MAX   10
#define RAND_MAX_TRY    20
#define HIS_LEN         20
#define ERR_OPEN_DIR   "Cannot open directory"
#define ERR_DIR_EMPTY  "Directory is empty"
#define IS_VALID_PTR(ptr) ( ((ptr) != NULL) && ((ptr) != (void*)-1) )

/* my usual type for array indexing */
typedef long uma;

/* common global variables and structs */
typedef struct list list;
struct list {
  char *path;
  char **item;
  uma first,last,sel,bold,nf,nd;
  list *(*create)(const char*);
  void (*kill)(list*);
  void (*keypress)(list*,int);
  void (*output)(list*);
  void (*play)(list*,uma);
  list *prev;
};

struct global {
  list *curr;
  int sigpipe[2];
  int ccted,editing,pausing;
  int list_height,show_hidden;
  int auto_play,repeat,shuffle,home_allinone;
  int width,height;
  pid_t player_pid;
  char *base_dir,*search_expr;
  char **player;
  char *search_string;
  char *error;
  terminal term;
  cct_textbox txt;
} G;

/* some prototypes, TODO: cleanup */
list *new_browser(const char*);
list *new_allinone(const char*);
void refresh_screen(void);
char *get_char(void);

/* the actual code */
#include "utils.c"
#include "myglob.c"
#include "lists.c"
#include "random.c"
#include "input.c"
#include "output.c"
#include "player.c"
#include "signals.c"
#include "browser.c"
#include "allinone.c"
#include "parse_arguments.c"
#include "run.c"

void init(int argc,char **argv) {
  list *n;

  static char *default_player[3] = { "mplayer","%f",NULL };
  G.ccted = 0;
  G.auto_play = 0;
  G.repeat = 0;
  G.curr = NULL;
  G.pausing = 0;
  G.player_pid = 0;
  G.show_hidden = 0;
  G.editing = 0;
  G.home_allinone = 0;
  G.search_string = NULL;
  G.player = (char**)default_player;
  G.error = NULL;
  parse_arguments(argc,argv);
  init_signals();
  n = ( G.home_allinone ? 
        new_allinone(G.base_dir) :
        new_browser(G.base_dir));
  if ( n == NULL ) end("%s: " ERR_OPEN_DIR,G.base_dir);
  if ( n == (void*)-1 ) end("%s: " ERR_DIR_EMPTY,G.base_dir);
  list_new(n);
  if ( cct_init_term(&G.term,1) == -1 ) end("cannot initialize cct");
  G.ccted = 1;
  G.width = G.term.w;
  G.height = G.term.h;
  if ( cct_textbox_init(&G.txt,&G.term,1,
                        G.height - 1,G.width - 1,0) == -1 )
  {
    end("cannot initialze cct's textbox");
  }
  G.list_height = G.height - 2;
  cct_icanon_off(&G.term);
  cct_echo_off(&G.term);
}

int main(int argc,char **argv) {
  init(argc,argv);
  main_loop();
  end(NULL);
  return 0;
}
