/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

struct input {
  char prompt,*dir,*display_dir,**entries;
  uma n,first,last,curr;
} I;

void input_init(char prompt) {
  cct_textbox_clear(&G.txt);
  I.prompt = prompt;
  I.curr = -1;
  G.editing = 1;
}

void input_output(void) {
  G.term.attr = 0;
  cct_putc(&G.term,0,G.height - 1,I.prompt);
  cct_textbox_draw(&G.txt);
}

int input_alphasort(const void *a,const void *b) {
  return strcmp(*(char**)a,*(char**)b);
}

uma input_scandir(const char *dir) {
  DIR *dd = opendir(dir);
  struct dirent *entry;
  void *p,*prev,*q = 0;
  char **i;
  uma n;
  
  n = 0;
  if ( !dd ) return -1;
  for( prev = 0 ; (entry = readdir(dd)) ; ) {
    if ( !hiddenfile(entry->d_name) ) {
      n++;
      p = malloc2(sizeof(void*) + strlen(entry->d_name) + 1);
      *(void**)p = 0;
      if ( prev ) *(void**)prev = p; else q = p;
      prev = p;
      p = (void**)p + 1;
      strcpy((char*)p,entry->d_name);
    }
  }
  closedir(dd);
  if ( n == 0 ) return -1;
  p = malloc2(n * sizeof(char*));
  i = p;
  for( ; q ; q = *(void**)q,i++ ) *i = (char*)((void**)q + 1);
  qsort(p,n,sizeof(char*),input_alphasort);
  if ( I.entries ) killdir(I.entries,I.n);
  I.entries = (char**)p;
  I.n = n;
  return 0;
}

int input_cmp_start(const void *key,const void *entry) {
  const char *k,*e;
  k = key;
  e = *(const char**)entry;
  for( ; k[0] && e[0] && k[0] == e[0] ; k++,e++ );
  if ( k[0] == 0 ) return 0;
  return (int)k[0] - (int)e[0];
}

void input_first_completion(void) {
  char *dir,*start,**first;
  uma i,j;
  if ( I.display_dir ) free(I.display_dir);
  /* get the directory to list */
  for( i = G.txt.last - 1 ; i >= 0 && G.txt.text[i] != '/' ; i-- );
  if ( i < 0 ) {
    dir = path_resolution(".");
    start = strdup2(G.txt.text);
    I.display_dir = 0;
  }
  else {
    char c;
    i++;
    c = G.txt.text[i];
    G.txt.text[i] = 0;
    dir = path_resolution(G.txt.text);
    I.display_dir = strdup2(G.txt.text);
    G.txt.text[i]= c;
    start = strdup2(&G.txt.text[i]);
  }
  /* check if the directory was already listed */
  if ( I.dir && !strcmp(dir,I.dir) ) {
    free(dir);
    goto input_find_first_last;
  }
  /* list the directory if possible */
  if ( input_scandir(dir) == -1 ) {
    I.curr = -1;
    free(start);
    return;
  }
  /* a new directory */
  if ( I.dir ) free(I.dir);
  I.dir = dir;
  /* find the first and last entry corresponding */
input_find_first_last:
  first = bsearch(start,I.entries,I.n,sizeof(char*),input_cmp_start);
  if ( !first ) {
    I.curr = -1;
    free(start);
    return;
  }
  i = (uma)(first - I.entries);
  j = i;
  for( ; i >= 0 && input_cmp_start(start,&I.entries[i]) == 0 ; i-- );
  I.first = i + 1;
  for( ; j < I.n && input_cmp_start(start,&I.entries[j]) == 0 ; j++ );
  I.last = j - 1;
  I.curr = I.first;
  free(start);
  /* display the first corresponding entry */
  cct_textbox_clear(&G.txt);
  if ( I.display_dir ) cct_textbox_insert(&G.txt,I.display_dir);
  cct_textbox_insert(&G.txt,I.entries[I.curr]);
}

void input_next_completion(void) {
  if ( I.curr == -1 ) return;
  I.curr++;
  if ( I.curr > I.last ) I.curr = I.first;
  cct_textbox_clear(&G.txt);
  if ( I.display_dir ) cct_textbox_insert(&G.txt,I.display_dir);
  cct_textbox_insert(&G.txt,I.entries[I.curr]);
}

void input_open(void) {
  int c;
  input_init(':');
  for(;;) {
    refresh_screen();
    c = cct_get_key(get_char());
    switch(c) {
      case '\n':
        G.editing = 0;
        return;
      case '\t':
        if ( I.curr == -1 ) {
          input_first_completion();
        }
        else {
          input_next_completion();
        }
        break;
      case 127:
      case 8:
        if ( G.txt.last <= 0 && G.txt.sel <= 0 ) {
          G.editing = 0;
          return;
        }
      default:
        I.curr = -1;
        cct_textbox_keypress(&G.txt,c);
        break;
    }
  }
}

char *get_pattern(const char *s) {
  uma n,len;
  char *buf;
  len = strlen(s);
  n = len + 3;
  buf = malloc2(n * sizeof(char));
  buf[0] = '*';
  memcpy(&buf[1],s,len);
  buf[len + 1] = '*';
  buf[len + 2] = 0;
  return buf;
}

void input_search(void) {
  int c;
  uma i;
  input_init('/');
  for(;;) {
    refresh_screen();
    c = cct_get_key(get_char());
    switch(c) {
      case '\n':
        if ( G.search_string ) free(G.search_string);
        G.search_string = get_pattern(G.txt.text);
        G.editing = 0;
        return;
      case CCT_KEY_UP:
      case 0x0D:
        i = list_search(G.curr,G.search_string,-1,G.curr->sel - 1);
        if ( i != -1 ) list_move_sel(G.curr,i);
        break;
      case CCT_KEY_DOWN:
      case 0x0F:
        i = list_search(G.curr,G.search_string,1,G.curr->sel + 1);
        if ( i != -1 ) list_move_sel(G.curr,i);
        break;
      case 127:
      case 8:
        if ( G.txt.last <= 0 && G.txt.sel <= 0 ) {
          G.editing = 0;
          return;
        }
      default:
        cct_textbox_keypress(&G.txt,c);
        if ( G.search_string ) free(G.search_string);
        G.search_string = get_pattern(G.txt.text);
        i = list_search(G.curr,G.search_string,1,G.curr->sel);
        if ( i != -1 ) list_move_sel(G.curr,i);
        break;
    }
  }
}

