/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void list_move_first(list *l) {
  if ( l->last < 0 ) {
    l->first = 0;
    l->bold = -1;
    l->sel = 0;
    return;
  }
  if ( l->sel < 0 ) l->sel = 0;
  if ( l->sel > l->last ) l->sel = l->last;
  if ( l->last < G.list_height ) {
    l->first = 0;
    return;
  }
  if ( l->first + G.list_height > l->last ) {
    l->first = l->last - G.list_height;
  }
  if ( l->sel < l->first ) {
    l->first = l->sel;
    return;
  }
  if ( l->sel > l->first + G.list_height ) {
    l->first = l->sel - G.list_height;
    return;
  }
}

void list_move_sel(list *l,uma sel) {
  l->sel = sel;
  list_move_first(l);
}

void list_keypress(list *l,int c) {
  switch(c) {
    case CCT_KEY_DOWN:
    case 'j':
      l->sel++;
      break;
    case CCT_KEY_UP:
    case 'k':
      l->sel--;
      break;
    case ' ':
      l->sel += G.list_height;
      break;
    case 'b':
      l->sel -= G.list_height;
      break;
    case 'g':
      l->sel = 0;
      break;
    case 'G':
      l->sel = l->last;
      break;
  }
  list_move_first(l);
}

uma list_search(list *l,const char *s,int direction,uma sel) {
  uma i;
  if ( sel < 0 ) sel = 0;
  if ( sel > l->last ) sel = l->last;
  if ( direction > 0 ) {
    for( i = sel ; i <= l->last ; i++ )
      if ( match(l->item[i],s,0) ) return i;
    for( i = 0 ; i <= sel ; i++ )
      if ( match(l->item[i],s,0) ) return i;
    return -1;
  }
  if ( direction < 0 ) {
    for( i = sel ; i >= 0 ; i-- )
      if ( match(l->item[i],s,0) ) return i;
    for( i = l->last ; i >= sel ; i-- )
      if ( match(l->item[i],s,0) ) return i;
    return -1;
  }
  return 0;
}

void list_kill(list *l) {
  if ( !IS_VALID_PTR(G.curr) ) return;
  if ( l->kill ) l->kill(l);
  free(l);
}

void list_clean_history(void) {
  list *l1,*l2;
  if ( !IS_VALID_PTR(G.curr) ) return;
  for( l1 = G.curr->prev ; l1 ; ) {
    l2 = l1->prev;
    list_kill(l1);
    l1 = l2;
  }
  G.curr->prev = NULL;
}

void list_refresh(void) {
  list *n;
  uma sel,first;
  if ( !G.curr->create ) return;
  sel = G.curr->sel;
  first = G.curr->first;
  n = G.curr->create(G.curr->path);
  n->first = first;
  list_move_sel(n,sel);
  n->prev = G.curr->prev;
  list_kill(G.curr);
  G.curr = n;
}

void list_new(list *l) {
  if ( !IS_VALID_PTR(l) ) return;
  l->prev = G.curr;
  G.curr = l;
}

void list_back(void) {
  list *n;
  n = G.curr->prev;
  if ( !n ) return;
  list_kill(G.curr);
  G.curr = n;
}

