/*
 * Copyright (C) 2012-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

int match_single(char c1,char c2,int cs) {
  if ( !cs ) {
    if ( c1 >= 'A' && c1 <= 'Z' ) c1 -= 'A' - 'a';
    if ( c2 >= 'A' && c2 <= 'Z' ) c2 -= 'A' - 'a';
  }
  if ( c1 != c2 ) return 0;
  return 1;
}

int match_range(char c,char r1,char r2,int cs) {
  const int cte = 'A' - 'a';
  if ( !cs ) {
    if ( (c >= 'a' && c <= 'z') ) {
      return (c >= r1 && c <= r2) ||
             ((c + cte >= r1) && (c + cte <= r2));
    }
    if ( (c >= 'A' && c <= 'Z') ) {
      return (c >= r1 && c <= r2) ||
             ((c - cte >= r1) && (c - cte <= r2));
    }
  }
  return c >= r1 && c <= r2;
}

uma match_bracket_return(uma r,uma len,int not) {
  if ( not ) return ( r ? 0 : len );
  return ( r ? len : 0 );
}

int match_bracket(const char c,const char *pattern,int cs) {
  uma not,len;
  const char *d;
  pattern++;
  /* we count the length of the bracket */
  len = 1;
  d = pattern;
  if ( d[0] == '!' ) {
    len++;
    d++;
  }
  if ( d[0] == ']' ) {
    len++;
    d++;
  }
  for( ; d[0] && d[0] != ']' ; d++,len++ );
  if ( d[0] == 0 ) return 0;
  len++;
  /* we now test whether c matches */
  if ( pattern[0] == '!' ) {
    not = 1;
    pattern++;
  }
  else {
    not = 0;
  }
  if ( pattern[0] == ']' ) {
    if ( c == ']' ) return match_bracket_return(1,len,not);
    pattern++;
  }
  for( ; pattern[0] != ']' ; ) {
    if ( pattern[1] == '-' && pattern[2] != ']' ) {
      if ( match_range(c,pattern[0],pattern[2],cs) ) {
        return match_bracket_return(1,len,not);
      }
      pattern += 3;
    }
    else {
      if ( match_single(c,pattern[0],cs) ) {
        return match_bracket_return(1,len,not);
      }
      pattern++;
    }
  }
  return match_bracket_return(0,len,not);
}

int match_rec(const char *b,const char *e,
              const char *pattern,int cs)
{
  for(;;) {
    switch(pattern[0]) {
      case '?':
        b++;
        pattern++;
        break;
      case '[': {
        uma r;
        r = match_bracket(b[0],pattern,cs);
        if ( !r ) return 0;
        b++;
        pattern += r;
        break;
      }
      case '*': {
        const char *t;
        for( ; pattern[0] == '*' ; pattern++ );
        if ( pattern[0] == 0 ) return 1;
        for( t = e ; t >= b ; t-- ) {
          if ( match_rec(t,e,pattern,cs) ) return 1;
        }
        return 0;
        break;
      }
      default:
        if ( !match_single(b[0],pattern[0],cs) ) return 0;
        b++;
        pattern++;
        break;
    }
  }
  return 1;
}

int match(const char *s,const char *pattern,int cs) {
  const char *b,*e;
  b = s;
  e = b + (strlen(s) - 1);
  return match_rec(b,e,pattern,cs);
}
