/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void do_pause(void) {
  if ( G.player_pid ) kill(G.player_pid,SIGSTOP);
}

void do_resume(void) {
  if ( G.player_pid ) kill(G.player_pid,SIGCONT);
}

void do_kill(void) {
  if ( G.player_pid ) kill(G.player_pid,SIGKILL);
}

void unbold_all(void) {
  G.curr->bold = -1;
}

void set_stop_flags(void) {
  G.player_pid = 0;
  G.pausing = 0;
  unbold_all();
}

void force_kill(void) {
  struct sigaction oact,act;
  pid_t pid;

  if ( !G.player_pid ) return;
  act.sa_handler = SIG_DFL;
  act.sa_flags = 0;
  sigemptyset(&act.sa_mask);
  if ( sigaction(SIGCHLD,&act,&oact) < 0 ) {
    end("unable to change signal handler for SIGCHLD");
  }
  kill(-G.player_pid,SIGKILL);
  pid = waitpid(G.player_pid,0,0);
  if ( pid != G.player_pid ) {
    end("something went wrong with the death of my child");
  }
  if ( sigaction(SIGCHLD,&oact,0) < 0 ) {
    end("unable to restore signal handler for SIGCHLD");
  }
  set_stop_flags();
}

void play(const char *file) {
  int i,in,newin,out,newout,err,newerr;
  pid_t pid;
  /* kill existing player */
  force_kill();
  /* fork */
  if ( (G.player_pid = fork()) ) return;
  /* create a new process group */
  pid = getpid();
  setpgid(pid,pid);
  /* restore to default for SIGCHLD and SIGWINCH */
  signal(SIGCHLD,SIG_DFL);
  signal(SIGWINCH,SIG_DFL);
  /* set file descriptors 0, 1 and 2 to be "/dev/null" */
  in = open(DEV_NULL,O_RDONLY);
  out = open(DEV_NULL,O_WRONLY);
  err = open(DEV_NULL,O_WRONLY);
  if ( in < 0 || out < 0 || err < 0 ) exit(EXIT_FAILURE);
  newin = dup2(in,0);
  newout = dup2(out,1);
  newerr = dup2(err,2);
  if ( newin < 0 || newout < 0 || newerr < 0 ) exit(EXIT_FAILURE);
  if ( close(in) < 0 || close(out) < 0 || close(err) < 0 ) {
    exit(EXIT_FAILURE);
  }
  /* replace all the ``%f" by file */
  for( i = 0 ; G.player[i] ; i++ ) {
    if ( !strcmp(G.player[i],"%f") ) G.player[i] = (char*)file;
  }
  /* launch the player */
  execvp(G.player[0],G.player);
  exit(EXIT_SUCCESS);
}

void what_next(void) {
  if ( !G.auto_play || !G.curr ) {
    set_stop_flags();
    refresh_screen();
    return;
  }
  if ( G.repeat ) {
    G.curr->play(G.curr,G.curr->bold);
    refresh_screen();
    return;
  }
  if ( G.shuffle ) {
    G.curr->bold = random2(G.curr);
  }
  else {
    G.curr->bold = (G.curr->bold + 1) % G.curr->nf;
  }
  G.curr->play(G.curr,G.curr->bold);
  refresh_screen();
}

#undef DEV_NULL

