/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void key_pressed(int c) {
  if ( IS_VALID_PTR(G.error) ) {
    free(G.error);
    G.error = NULL;
  }
  switch(c) {
    case 4: /* CTRL+D */
    case 'q':
      force_kill();
      end(NULL);
      return;
    case 'x':
      force_kill();
      unbold_all();
      return;
    case 'p':
      if ( !G.player_pid ) return;
      G.pausing = ( G.pausing ? 0 : 1 );
      if ( G.pausing ) do_pause(); else do_resume();
      return;
    case '=':
      screen_resized();
      return;
    case 'v':
      do_kill();
      return;
    case 's':
      G.shuffle = ( G.shuffle ? 0 : 1 );
      if ( G.shuffle ) random_init();
      return;
    case 'a':
      G.auto_play = ( G.auto_play ? 0 : 1 );
      return;
    case 'r':
      G.repeat = ( G.repeat ? 0 : 1 );
      return;
    case 'i':
      list_move_sel(G.curr,G.curr->bold);
      return;
    case 18: /* CTRL+R */
    case '5':
      list_refresh();
      return;
    case 8:
    case 127:
      list_back();
      return;
    case 'h':
      list_new(new_browser(G.base_dir));
      return;
    case ':': {
      char *s;
      struct stat buf;
      input_open();
      if ( G.txt.last <= 0 && G.txt.sel <= 0 ) return;
      s = path_resolution(G.txt.text);
      if ( stat(s,&buf) != -1 && S_ISDIR(buf.st_mode) ) {
        list_new(new_browser(s));
        free(G.base_dir);
        G.base_dir = strdup2(s);
      }
      free(s);
      return;
    }
    case '/':
      input_search();
      return;
    case 'n': {
      uma i;
      if ( !IS_VALID_PTR(G.search_string) ) return;
      i = list_search(G.curr,G.search_string,1,G.curr->sel + 1);
      if ( i != -1 ) list_move_sel(G.curr,i);
      return;
    }
    case 'N': {
      uma i;
      if ( !IS_VALID_PTR(G.search_string) ) return;
      i = list_search(G.curr,G.search_string,-1,G.curr->sel - 1);
      if ( i != -1 ) list_move_sel(G.curr,i);
      return;
    }
  }
  if ( G.curr->keypress ) G.curr->keypress(G.curr,c);
}

char *get_char(void) {
  int r;
  fd_set rfds;

  for(;;) {
    FD_ZERO(&rfds);
    FD_SET(0,&rfds);
    FD_SET(G.sigpipe[0],&rfds);
    r = select(max(0,G.sigpipe[0]) + 1,&rfds,0,0,0);
    if ( r == -1 ) continue;
    if ( FD_ISSET(G.sigpipe[0],&rfds) ) {
      char c;
      if ( read(G.sigpipe[0],&c,1) <= 0 ) continue;
      if ( c == 't' ) screen_resized();
      if ( c == 'c' ) manage_sigchld();
    }
    if ( FD_ISSET(0,&rfds) ) return cct_input(0);
  }
  return 0;
}

void main_loop(void) {
  int c;
  for(;;) {
    refresh_screen();
    c = cct_get_key(get_char());
    if ( c == 0 ) continue;
    key_pressed(c);
  }
}

