/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void manage_sigchld(void) {
  pid_t pid;
  int status;
  if ( !G.player_pid ) return;
  pid = waitpid(G.player_pid,&status,WCONTINUED | WUNTRACED);
  if ( pid != G.player_pid ) {
    end("something went wrong with death of my child");
  }
  if ( WIFSTOPPED(status) && WSTOPSIG(status) == SIGSTOP ) {
    G.pausing = 1;
    return;
  }
  else if ( WIFCONTINUED(status) ) {
    G.pausing = 0;
    return;
  }
  else {
    G.player_pid = 0;
    what_next();
  }
}

void write_sigpipe(char c) {
  int try,w;
  for( try = 10 ; try > 0 ; try-- ) {
    w = write(G.sigpipe[1],&c,1);
    if ( w == 1 ) return;
    if ( w == 0 || (w == -1 && errno == EINTR) ) continue;
    break;
  }
  end("cannot write to signals pipe (this is experimental !)");
}

void handle_sigchld(int s) {
  if ( s != SIGCHLD ) return;
  write_sigpipe('c');
}

void handle_sigwinch(int s) {
  if ( s != SIGWINCH ) return;
  write_sigpipe('t');
}

void init_signals(void) {
  struct sigaction act;
  if ( pipe(G.sigpipe) < 0 ) {
    end("cannot create signals pipe (this is experimental !)");
  }
  act.sa_handler = handle_sigchld;
  act.sa_flags = 0;
  sigemptyset(&act.sa_mask);
  sigaddset(&act.sa_mask,SIGCHLD);
  sigaddset(&act.sa_mask,SIGWINCH);
  if ( sigaction(SIGCHLD,&act,0) < 0 ) {
    end("cannot set handler for SIGCHLD (this is experimental !)");
  }
  act.sa_handler = handle_sigwinch;
  if ( sigaction(SIGWINCH,&act,0) < 0 ) {
    end("cannot set handler for SIGWINCH (this is experimental !)");
  }
}

